use crate::db::*;

use sea_orm::{sea_query::IntoCondition, *};

#[derive(Deserialize)]
pub struct Filter {
    name: Option<String>,
}

impl IntoCondition for Filter {
    fn into_condition(self) -> Condition {
        Condition::all().add_option(
            self.name
                .map(|name| distro::Column::Name.like(format!("%{}%", name))),
        )
    }
}

pub async fn page(
    conn: &DbConn,
    per_page: u64,
    page: u64,
    filter: Filter,
) -> Result<Vec<distro::Model>> {
    let paginator = Distro::find()
        .filter(filter)
        .order_by_asc(distro::Column::Id)
        .paginate(conn, per_page);
    let repos = paginator.fetch_page(page).await?;

    Ok(repos)
}

pub async fn by_id(conn: &DbConn, id: i32) -> Result<Option<distro::Model>> {
    distro::Entity::find_by_id(id).one(conn).await
}

pub async fn insert(conn: &DbConn, name: &str, description: Option<&str>) -> Result<distro::Model> {
    let model = distro::ActiveModel {
        id: NotSet,
        name: Set(String::from(name)),
        description: Set(description.map(String::from)),
    };

    model.insert(conn).await
}
