use crate::db::*;

use sea_orm::{sea_query::IntoCondition, *};

#[derive(Deserialize)]
pub struct Filter {
    name: Option<String>,
}

impl IntoCondition for Filter {
    fn into_condition(self) -> Condition {
        Condition::all().add_option(
            self.name
                .map(|name| repo::Column::Name.like(format!("%{}%", name))),
        )
    }
}

pub async fn page(
    conn: &DbConn,
    per_page: u64,
    page: u64,
    filter: Filter,
) -> Result<Vec<repo::Model>> {
    let paginator = Repo::find()
        .filter(filter)
        .order_by_asc(repo::Column::Id)
        .paginate(conn, per_page);
    let repos = paginator.fetch_page(page).await?;

    Ok(repos)
}

pub async fn by_id(conn: &DbConn, id: i32) -> Result<Option<repo::Model>> {
    repo::Entity::find_by_id(id).one(conn).await
}

pub async fn by_name(conn: &DbConn, name: &str) -> Result<Option<repo::Model>> {
    Repo::find()
        .filter(repo::Column::Name.eq(name))
        .one(conn)
        .await
}

pub async fn insert(
    conn: &DbConn,
    distro_id: i32,
    name: &str,
    description: Option<&str>,
) -> Result<repo::Model> {
    let model = repo::ActiveModel {
        id: NotSet,
        distro_id: Set(distro_id),
        name: Set(String::from(name)),
        description: Set(description.map(String::from)),
    };

    model.insert(conn).await
}
