pub struct Chunked<I> {
    iter: I,
    chunk_size: usize,
}

impl<I: Iterator> Chunked<I> {
    pub fn new<T: IntoIterator<IntoIter = I>>(into: T, chunk_size: usize) -> Self {
        Self {
            iter: into.into_iter(),
            chunk_size,
        }
    }
}

// https://users.rust-lang.org/t/how-to-breakup-an-iterator-into-chunks/87915/5
impl<I: Iterator> Iterator for Chunked<I> {
    type Item = Vec<I::Item>;

    fn next(&mut self) -> Option<Self::Item> {
        Some(self.iter.by_ref().take(self.chunk_size).collect())
            .filter(|chunk: &Vec<_>| !chunk.is_empty())
    }
}
